function x = bsp(dm, p, C),

% BSP Implement an (Open) 2-d Non-periodic b-spline with 
% uniform internal knots.
%
% BSP(dm, p, C). The B-Spline is represented by,  
%
%             N, M 
%            -------
%  Z(X, Y) = >>  C(I, J) B(I, J ; PI, PJ)(X, Y) 
%            -------
%            I=0, J=0
% where 
%       B(I, J; PI, PJ)(X, Y) = B(I, PI)(X)*B(J, PJ)(Y)
% 
% The variable dm is a 4 element row vector that gives the x and y support 
% respectively [ xmin xmax ymin ymax ].   The degree is given by p, whilst 
% k is order (p + 1). The control matrix is specified thus-ly, 
% 
%           [ C(0,0) ... C(0,M)  ]
%           [        .           ]
%       C = [         .          ]
%           [          .         ]
%           [ C(N,0) ... C(N, M) ]
%
% The number of control points will be (N+1)x(M+1). The (PI, PJ) vectors
% are determined by BSP1DNPKNTS (Non-periodic b-spline with uniform internal 
% knots).
% 
%
% Example, 
%
%  C  = ones(7); 
%  dm = [ -1 1 -1 1 ]; p  = [ 3 3 ]; % basis functions are x^3         
%  y = bsp(dm, p, C);
%  get(y, 0.5, 0.5)
% 
% NOTES,
%   
% 05/03/07 modified to be consistent with mathworld definition replaced k (order) 
% with degree (p) :
% 
%   Weisstein, Eric W. "B-Spline." From MathWorld--A Wolfram Web Resource. 
%   http://mathworld.wolfram.com/B-Spline.html 
%


%
% support limits
% 
% u(k) <= u <= u(n+1-k)
% k = 1 ... n+1
% 
% (n+1) = length(C) = number of control points (entire b-spline) 


x = init_bsp;

if  (nargin == 0),
    return 
else 
    x.coeff = C;
end

x.family    = 'B-Spline';
x.type      = '2-D Cartesian';
x.coord     = 'cartesian';
x.support   = dm; 
x.m         = size(x.coeff,1)-1;
x.n         = size(x.coeff,2)-1;
x.order     = p+1;
x.degree    = p;
x.kt{1}     = bsp1dnpknts(x.support(1:2), x.n, p(1));  % knot seq. for u 
x.kt{2}     = bsp1dnpknts(x.support(3:4), x.m, p(2));  % knot seq. for v 
return

function x = init_bsp,
    x.coeff  = []; % ones([ 8 8 ]);  % default b-spline
    x.family = []; 
    x.type   = [];
    x.m      = []; 
    x.n      = []; 
    x.order  = [];
    x.degree = [];
    x.kt{1}  = []; % knot sequence for u 
    x.kt{2}  = []; % knot sequence for v 
    x.coord  = []; % 'cartesian';
    x.support = [];
    x  = class(x, 'bsp');
return 
