function [z, varargout] = conic(x, y, CV, CC, str),

% CONIC Define a conic along the z-axis with apex at the origin. A conic is described 
% by,
%                CV*r^2
%   z = -----------------------------
%       1 + sqrt(1 - CV^2*(CC+1)*r^2)
% where     
%        r  = radial distance (i.e., sqrt(x^2+y^2);
%        CV = 1/R is the curvature and R is the radius of curvature 
%        CC = p-1 is the conic constant and p is the conic p value
% 
% The conic constant determines the shape of the conic, i.e., 
%               CC < -1     hyperboloid,
%               CC = -1     paraboloid, 
%               -1 < CC < 0 ellipsoid 
%               CC = 0      sphere, 
% 
% CONIC(x, y, CV, CC, str). The pair (x, y) is the position in the (x, y) plane, 
% CV=1/R is the curvature, CC is the conic constant.  The optional parameter str 
% is { 'z', 'dzdx', 'dzdy','normal' }.  If strf is omitted, the height ('z') will be 
% returned. 
%
% [R, nArgList ] = CONIC('default').  Returns information for the RAYTRACE 
% toolbox, namely a default conic surface, along with the argument list.
%
% f = CONIC('inline').  Returns an inline version for the depth function ('z')   .
%
% Examples,
%   
%   nArgs = conic('default');
%   z = conic(1, 1.5, nArgs{:})
%   z = conic('inline'); z(1, 1.5, nArgs{:})
% 
% Notes : 
%
% The OSLO ray-tracing documentation seems to contain a small error. The CV term 
% is to the power of 1 only, it should be to the power of two. 


if  ((nargin == 1)&(ischar(x))),
    switch(x),
        case { 'default' }
             z = { 1/7.87, -0.28 }; 
            varargout = { {'CV', 'CC'} };
        case { 'inline' },
            % bug 15/3/06 - added squares to the r's [DOES THAT AFFECT PAPER]
            % z   = inline('(CV*r^2)/(1 + sqrt(1 - (CV^2)*(CC+1)*r^2))','r','CV','CC');
            z = '(CV*rx^2)/(1 + sqrt(1 - (CV^2)*(CC+1)*rx^2))';
            z = strrep(z,'rx','(sqrt(x^2+y^2))');
            z = inline(z,'x','y','CV','CC');      
        otherwise,
            error('(CONIC) Unknown invocation parameter.');
    end
    return
end


s2  = x.^2 + y.^2;
z   = (CV*s2)./(1 + sqrt(1 - CV^2*(CC+1)*s2));

% as implemented by Sicam et al.
k   = CC+1;
R   = 1/CV;
        
if (nargin == 4),
    return
end 


switch(str)
    case 'z',
        return
    case 'dzdx',
        z = x./(R - k*z);
        return
    case 'dzdy',
        z = y./(R - k*z);
        return
    case 'normal',
        z0      = z;
        z(:,1)  = x./(R - k*z0);
        z(:,2)  = y./(R - k*z0);
        z(:,3)  = -ones(size(z0));        
        return
    otherwise,
        error([ 'unknown argument ''' str ''' passed to conic.m' ]);
end 

return
