function y = bhwTerm(r, phi, p),

% BHWTERM Bhatia-Wolf term function (L,n)
%
%
%   y = bhwTerm(r, phi, p),
%
%
%                      { N^m_n T^m_n cos(m*theta) even p, m ~= 0
%   B^m_n(p, theta) =  { N^m_n T^m_n sin(m*theta) odd p,  m ~= 0
%                      { N^m_n T^0_n                      m = 0
%
% where 
%       n           is the radial order
%       m           is the azimuthal frequency
%       p           is the polynomial-ordering number 
%
%                   n = ceil(sqrt(p)) - 1
%                   m = floor({p-[(n-1)^2+2(n-1)+1]}/2)
%
%       N^m_n       is the normalization factor,
%
%                   sqrt(2*(n+1)/(1+delta)) where delta is delta(m) the
%                   kronecker delta function
%
%       T^m_n(rho)  is given by,
%
%
%                 n-m
%                 ---    (-1)^s (2*n+1-s)! 
%   T^m_n(rho)  =  >    ------------------------ rho^(n-s)
%                 ---   s! (n-m-s)! (n+m+1-s) ! 
%                 s = 0
% 
% where 
%           rho = distance in unit circle
%           phi = radian angle
%           m   = order
%           n   = order
%
% note:
%       0 <= rho <= 1
%       0 <= phi < 2*pi
%
%
% implements Bhatia=Wolf polynomials as described in : 
%
% Modeling of Corneal Surfaces with Radial Polynomials. Iskander, DR;
% Morelande, MR; Collins MJ; Davis, B. IEEE Trams. Biomed. Eng. 49(4) April
% 2002. 


if (p==1),
   y = ones(size(r));
   
   % zero outside aperture 
   y(abs(r) > 1) = 0;

   return
end 

% single indexing to double 
[n,m] = bhwSgl2Dbl(p);

% normalization 
if  (m==0),
    N_mn = sqrt(n+1);
else
    N_mn = sqrt(2*(n+1));
end


% create radial polynomial 
for s = 0:(n-m)
    r0(:,s+1) = r.^(n-s);
end
T_mn = r0*bhwRadialCoeff(n, m);



% azimuthal partbasisOSAstring
if (m ~= 0),    

 % angular part 
   if  (mod(p,2)),
       
        % odd
        y = N_mn.*T_mn.*sin(m*phi);   
    else
        % even
        y = N_mn.*T_mn.*cos(m*phi);   
   end

else
    y = N_mn.*T_mn;
end

% zero outside aperture 
y(abs(r) > 1) = 0;
    
return

