function y = znkBasisOSADv(r, phi, n, m, strf, repl),

% ZNKBASISOSADV derivative of zernike basis (OSA) function (L,n)
%
% ZNKBASISOSADV(r, phi, n, m, <'rho','phi'>).  Here r is the radial 
% distance, angular distance phi and (n,m) is the term.  Then strf 
% is the { 'rho','phi' } and repl is the value to use outside the 
% aperture. 
% 
% NOTE: 
%       0 <= r <= 1
%       0 <= phi < 2*pi
%       n >= m >= 0
%
% Thibos, LN, Applegate, RA, Schwiegerling, JT, Webb, R. (2000).   
% Standards for reporting the optical Aberrations of Eyes.  
% In: Lakminarayan, V., ed. Trends in Optics and Photonics, Vision Science and Its Applications, 
% OSA Technical Digest Series, Washington, DC: Optical Society of America, Vol. 35:232-244.
% 
% Jim Schwiegerling, John E. Greivenkamp and Joseph M. Miller
% Representation of videokeratoscopic of height data with Zernike
% polynomials. Vol. 12, No. 10/ October 1995/J. Opt. Soc. Am. A


% 11/8 modified to avoid use of an 'abs(m)'.  Replaced with m instead.


% return 'repl' variable for out-of-bounds co-ordinates 
if  (nargin < 6), 
    repl = 0;
end

% return and m=0, n=0 term
if ((m==0)&(n==0))
        y = zeros(size(r));  
        i = find(r>1);
        y(i) = repl;
        return
end

% go to hardcoded
% y = znkQuickBasis(n, m, strf);
% if (~isempty(y)),
%   y = inline(vectorize(y),'r','phi');
%   y = y(r, phi);
%   i = find(r > 1);
%   y(i) = repl;
%   return
% end

% term not listed


% manual method
    delta = inline('n==0');
    N_mn = sqrt((2*(n+1))/(1+delta(m)));
    
    R_mn = 0;
    for s=0:(n-abs(m))/2,
   
        % each zernike term 

        switch(strf),
            case { 'phi', 'dzdt' },
                nm      = (-1)^s*factorial(n-s)*r.^(n-2*s);
            case { 'rho', 'dzdr' },
                if (n-2*s == 0), 
                    nm      = zeros(size(r));
                else
                    nm      = (-1)^s*factorial(n-s)*(n-2*s)*r.^(n-2*s-1);    
                end
            otherwise,
                error('(ZNKBASISOSADV) Unknown derivative.');
        end
        
            
        l = 0.5*(n+abs(m))-s;
        if (isequal(floor(l),l))
            dm      = factorial(s)*factorial(0.5*(n+abs(m))-s)*factorial(0.5*(n-abs(m))-s);
        else 
            warning([ 'fractional factorial! n = ' num2str(n) ', m = ' num2str(m) ]); 
        end
                    
        R_mn    = R_mn + nm/dm;
    
    end
    
    % angular part 
    switch(strf),
        case { 'rho', 'dzdr' },
            if  ((m) >= 0),
                y = N_mn.*R_mn.*cos(m*phi);     % 11/8 removed the abs(m) -> m  
            else
                y = -N_mn.*R_mn.*sin(m*phi);    % 11/8 removed the abs(m) -> m
            end
        case { 'phi', 'dzdt' },
            if  ((m) >= 0),
                y = -m*N_mn.*R_mn.*sin(m*phi);  % 11/8 removed the abs(m) -> m   
            else
                y = -m*N_mn.*R_mn.*cos(m*phi);  % 11/8 removed the abs(m) -> m 
            end
        otherwise,
         error('(ZNKBASISOSADV) Unknown derivative.');
    end
    
    
    y(abs(r)>1) = repl;    
return

